@@ -4,10 +4,10 @@ from __future__ import annotations as _annotations
 
 from functools import partial, partialmethod
 from types import FunctionType
-from typing import TYPE_CHECKING, Any, Callable, TypeVar, Union, overload
+from typing import TYPE_CHECKING, Any, Callable, Literal, TypeVar, Union, overload
 from warnings import warn
 
-from typing_extensions import Literal, Protocol, TypeAlias, deprecated
+from typing_extensions import Protocol, TypeAlias, deprecated
 
 from .._internal import _decorators, _decorators_v1
 from ..errors import PydanticUserError
@@ -19,30 +19,24 @@ _ALLOW_REUSE_WARNING_MESSAGE = '`allow_reuse` is deprecated and will be ignored;
 if TYPE_CHECKING:
 
     class _OnlyValueValidatorClsMethod(Protocol):
-        def __call__(self, __cls: Any, __value: Any) -> Any:
-            ...
+        def __call__(self, __cls: Any, __value: Any) -> Any: ...
 
     class _V1ValidatorWithValuesClsMethod(Protocol):
-        def __call__(self, __cls: Any, __value: Any, values: dict[str, Any]) -> Any:
-            ...
+        def __call__(self, __cls: Any, __value: Any, values: dict[str, Any]) -> Any: ...
 
     class _V1ValidatorWithValuesKwOnlyClsMethod(Protocol):
-        def __call__(self, __cls: Any, __value: Any, *, values: dict[str, Any]) -> Any:
-            ...
+        def __call__(self, __cls: Any, __value: Any, *, values: dict[str, Any]) -> Any: ...
 
     class _V1ValidatorWithKwargsClsMethod(Protocol):
-        def __call__(self, __cls: Any, **kwargs: Any) -> Any:
-            ...
+        def __call__(self, __cls: Any, **kwargs: Any) -> Any: ...
 
     class _V1ValidatorWithValuesAndKwargsClsMethod(Protocol):
-        def __call__(self, __cls: Any, values: dict[str, Any], **kwargs: Any) -> Any:
-            ...
+        def __call__(self, __cls: Any, values: dict[str, Any], **kwargs: Any) -> Any: ...
 
     class _V1RootValidatorClsMethod(Protocol):
         def __call__(
             self, __cls: Any, __values: _decorators_v1.RootValidatorValues
-        ) -> _decorators_v1.RootValidatorValues:
-            ...
+        ) -> _decorators_v1.RootValidatorValues: ...
 
     V1Validator = Union[
         _OnlyValueValidatorClsMethod,
@@ -125,7 +119,7 @@ def validator(
 
     if allow_reuse is True:  # pragma: no cover
         warn(_ALLOW_REUSE_WARNING_MESSAGE, DeprecationWarning)
-    fields = tuple((__field, *fields))
+    fields = __field, *fields
     if isinstance(fields[0], FunctionType):
         raise PydanticUserError(
             '`@validator` should be used with fields and keyword arguments, not bare. '
@@ -171,8 +165,7 @@ def root_validator(
 ) -> Callable[
     [_V1RootValidatorFunctionType],
     _V1RootValidatorFunctionType,
-]:
-    ...
+]: ...
 
 
 @overload
@@ -185,8 +178,7 @@ def root_validator(
 ) -> Callable[
     [_V1RootValidatorFunctionType],
     _V1RootValidatorFunctionType,
-]:
-    ...
+]: ...
 
 
 @overload
@@ -200,8 +192,7 @@ def root_validator(
 ) -> Callable[
     [_V1RootValidatorFunctionType],
     _V1RootValidatorFunctionType,
-]:
-    ...
+]: ...
 
 
 @deprecated(
